#!/usr/bin/python

import sys
import pickle
import getopt
from scapy.all import *
from network_flow import *
from flow_feature import  *

def main(argv):


    port_nums = []
    for x in range(1,65536):
      port_nums.append({'UDP' : '', 'TCP' : '' })

    port_nums[22] = {'UDP': '', 'TCP':'SSH'}
    port_nums[53] = {'UDP' : 'DNS','TCP' : 'DNS' }
    port_nums[137] = {'UDP': 'NetBIOS NS', 'TCP':'NetBIOS NS'}
    port_nums[138] = {'UDP': 'NetBIOS DS', 'TCP':'NetBIOS DS'}
    port_nums[1196] = {'UDP': 'NETMAGIC', 'TCP':''}
    port_nums[1900] = {'UDP': 'MS SSDP', 'TCP':'MS SSDP'}
    port_nums[5351] = {'UDP': 'NAT PORT MAPPING', 'TCP':'NAT PORT MAPPING'}
    port_nums[17500] = {'UDP': 'DROPBOX DISCOVERY', 'TCP':'DROPBOX PROTOCOL'}

    pklfile = ""
    try:
      opts, args = getopt.getopt(argv, "hf:p:", ["help", "pfile=", "pklfile="])
    except(getopt.GetoptError):
      usage()
      sys.exit(2)

    for opt, arg in opts:
      if opt in ("-h", "--help"):
        usage()
        sys.exit()
      if opt in ("-f"):
        pfile = arg
      if opt in ("-p"):
        pklfile = arg

    fl = FlowsList()

    try:
      pcktList = rdpcap(pfile)
    except Exception as e:
      print "Please provide the pcap file"
      usage()
      return False

    try:
      pkl_file = open(pklfile, 'rb')
    except Exception as e:
      print "Please Provide the pickle file"
      usage()
      return False

    appflowmap = pickle.load(pkl_file)

    try:
      pkl_file = open(pklfile+"2", 'rb')
    except Exception as e:
      print "Please Provide the pickle file 2"
      usage()
      return False

    # Internet address of the host
    inet_addr = pickle.load(pkl_file)
#    print inet_addr

    #pcktList.summary()
    for frame_p in pcktList:
      packettype = str(type(frame_p.__dict__['payload']))

      if packettype == "<class 'scapy.layers.inet.IP'>" or \
         packettype == "<class 'scapy.layers.inet6.IPv6'>":

        if packettype == "<class 'scapy.layers.inet.IP'>":
          packet_p = frame_p.__dict__['payload'][IP]
        else:
          packet_p = frame_p.__dict__['payload'][IPv6]

        trprt_layer = str(packet_p.payload.__class__)
        if trprt_layer == "<class 'scapy.layers.inet.TCP'>" :
          tcp_p = packet_p['TCP']
          if str(packet_p.src) in inet_addr:
            temp_flow = NetworkFlow(packet_p.src, tcp_p.sport, packet_p.dst, tcp_p.dport, "TCP", frame_p.time)
            temp_flow.append_packet_client({'packet':packet_p, 'time':frame_p.time})
            fl.append_flow(temp_flow)
          elif str(packet_p.dst) in inet_addr:
            temp_flow = NetworkFlow(packet_p.dst, tcp_p.dport, packet_p.src, tcp_p.sport, "TCP", frame_p.time)
            temp_flow.append_packet_server({'packet':packet_p, 'time':frame_p.time})
            fl.append_flow(temp_flow)
          #print "test:", temp_flow.is_packet_in_flow(packet_p.src, tcp_p.sport, packet_p.dst, tcp_p.dport, "TCP", frame_p.time)
          #print "test:", temp_flow.is_packet_in_flow(packet_p.dst, tcp_p.dport, packet_p.src, tcp_p.sport, "TCP", frame_p.time)

        elif trprt_layer == "<class 'scapy.layers.inet.UDP'>" :
          udp_p = packet_p['UDP']
          if str(packet_p.src) in inet_addr:
            temp_flow = NetworkFlow(packet_p.src, udp_p.sport, packet_p.dst, udp_p.dport, "UDP", frame_p.time)
            temp_flow.append_packet_client({'packet':packet_p, 'time':frame_p.time})
            fl.append_flow(temp_flow)
          elif str(packet_p.dst) in inet_addr:
            temp_flow = NetworkFlow(packet_p.dst, udp_p.dport, packet_p.src, udp_p.sport, "UDP", frame_p.time)
            temp_flow.append_packet_server({'packet':packet_p, 'time':frame_p.time})
            fl.append_flow(temp_flow)

        elif trprt_layer == "<class 'scapy.layers.inet.ICMP'>" :
          icmp_p = packet_p['ICMP']

          if str(packet_p.src) in inet_addr:
            temp_flow = NetworkFlow(packet_p.src, icmp_p.sport, packet_p.dst, icmp_p.dport, "UDP", frame_p.time)
            temp_flow.append_packet_client({'packet':packet_p, 'time':frame_p.time})
            fl.append_flow(temp_flow)
          elif str(packet_p.dst) in inet_addr:
            temp_flow = NetworkFlow(packet_p.dst, icmp_p.dport, packet_p.src, icmp_p.sport, "UDP", frame_p.time)
            temp_flow.append_packet_server({'packet':packet_p, 'time':frame_p.time})
            fl.append_flow(temp_flow)

#        else:
#          print "This type of packets are not supported:" + trprt_layer + "\n\t" + str(frame_p.__dict__['payload'].__str__) +"\n\n\n"

      #else:
        #print "Non IP packets are not supported:" + packettype + "\n\t" + str(frame_p.__dict__['payload'].__str__) +"\n\n\n"

    for flow in fl.flows:
      for app in appflowmap:
        if(flow.is_matching_flows(app['src'], app['src_port'], app['dst'], app['dst_port'], app['protocol'].upper())):
          flow.appname = app['appname']
          #appflowmap.remove(app)
          break

      if(flow.appname == ""):

        flow.appname = port_nums[flow.source_port][flow.protocol]

        if flow.appname == "":
          flow.appname = port_nums[flow.target_port][flow.protocol]
          
        if flow.appname == "":
          flow.appname = "UNKNOWN"


    for flow in fl.flows:
      flow.features = Flow_Feature(flow)

    for flow in fl.flows:
      print str(flow.features) + ','

 #   print fl

def usage():
    print "Usage: python flow_capture.py -f [PCAP File] -p [Netstat Pickle File]"
    print

if __name__ == "__main__":
    main(sys.argv[1:])
