#!/usr/bin/python

# This script extract a dictionary containing some of the common ML features for the given flows.
import time
from math import sqrt

class Flow_Feature:
  def __init__(self, flow):
    self.flow = flow
    self.features = {'appname':flow.appname, 'source port':flow.source_port, 'target port':flow.target_port, 'source ip':flow.source_ip, \
      'target_ip':flow.target_ip, 'protocol':flow.protocol, \
      'num client packets': len(flow.clientpackets), 'num server packets': len(flow.serverpackets), 'num packets': len(flow.clientpackets) + len(flow.serverpackets)}

    self.ff_flowduration()

    packets = self.flow.clientpackets
    self.ff_inter_arrival(packets, 'client')
    self.ff_packetsize(packets, 'client')

    packets = self.flow.serverpackets
    self.ff_inter_arrival(packets, 'server')
    self.ff_packetsize(packets, 'server')


  def merge_featuress(self, f1, f2,ftype):
    return f1

  def __str__(self):
    return str(self.features)


  # Calculates the flow duration each flow
  def ff_flowduration(self):

    cpackets = self.flow.clientpackets
    spackets = self.flow.serverpackets

    c_start = float('inf')
    c_end   = 0

    s_start = float('inf')
    s_end   = 0

    if len(cpackets) > 0:
      c_start = cpackets[0]['time']
      c_end   = cpackets[-1]['time']

    if len(spackets) > 0:
      s_start = spackets[0]['time']
      s_end   = spackets[-1]['time']

    start = min(c_start, s_start)
    end = max(c_end, s_end)
    self.features['flow duration'] =  end - start

  # Calculates the message size for each flow (mean, std, max, min)
  def ff_packetsize(self,  packets, keyval):
    n = len(packets)

    if n == 0 :
      self.features[keyval + ' avg packet size'] = 0
      self.features[keyval + ' std packet size'] = 0
      self.features[keyval + ' max packet size'] = 0
      self.features[keyval + ' min packet size'] = 0
      return

    sum_size = 0
    min_size = float('inf')
    max_size = 0

    psize = list()

    for i in range(n):
      psize.append(packets[i]['packet'].len)

    for i in range(n):
      s = psize[i]

      if( s > max_size):
        max_size = s

      if( s < min_size):
        min_size = s

      sum_size += s

    mean_size = sum_size / n

    sum_size = 0
    for i in range(n):
      sum_size += (psize[i] - mean_size) ** 2

    std_size = sqrt(sum_size / n)

    self.features[keyval + ' avg packet size'] = mean_size
    self.features[keyval + ' std packet size'] = std_size
    self.features[keyval + ' max packet size'] = max_size
    self.features[keyval + ' min packet size'] = min_size

    return


  # Calculates the interarrival times (mean, std, max, min)
  def ff_inter_arrival(self, packets, keyval):
    n = len(packets) -1

    if n <= 0 :
      self.features[keyval + ' avg interarrival time'] = 0
      self.features[keyval + ' std interarrival time'] = 0
      self.features[keyval + ' max interarrival time'] = 0
      self.features[keyval + ' min interarrival time'] = 0
      return

    sum_time = 0
    min_time = float('inf')
    max_time = 0

    iatime = list()

    for i in range(n):
      iatime.append(packets[i+1]['time'] - packets[i]['time'])

    for i in range(n):
      diff = iatime[i]

      if( diff > max_time):
        max_time = diff

      if( diff < min_time):
        min_time = diff

      sum_time += diff

    if n != 0:
      mean_time = sum_time / n
    else:
      mean_time = sum_time
      min_time = mean_time
      max_time = mean_time

    sum_time = 0
    for i in range(n):
      diff = iatime[i]
      sum_time += (diff - mean_time) ** 2

    if n != 0:
      std_time = sqrt(sum_time / n)
    else:
      std_time = 0

    self.features[keyval + ' avg interarrival time'] = mean_time
    self.features[keyval + ' std interarrival time'] = std_time
    self.features[keyval + ' max interarrival time'] = max_time
    self.features[keyval + ' min interarrival time'] = min_time


    return
