#!/usr/bin/python

import time
import math
from socket import *

# This class given a 5-tuple packet features, it generates the network flow object
class NetworkFlow:
  def __init__(self, source_ip, source_port, target_ip, target_port, protocol, timestamp,timeout = 100, appname = ""):
    self.appname = appname
    self.source_ip = source_ip
    self.source_port = source_port
    self.target_ip = target_ip
    self.target_port = target_port
    self.protocol = protocol.upper()
    self.starttimestamp = timestamp
    self.timestamp = timestamp
    self.clientpackets = []
    self.serverpackets = []
    self.timeout = timeout
    self.features = {}

  def is_matching_flows(self, source_ip, source_port, target_ip, target_port, protocol):
    ip_matched = False
    ports_matched = False
    protocol_matched = False

#    print "========================================================"
#    print self
#    print source_ip, source_port, target_ip, target_port, protocol

    if(self.protocol ==  protocol):
      protocol_matched = True
    else:
      return False

#    print ip_matched, ports_matched, protocol_matched

    if (source_ip == '0.0.0.0' and target_ip == '0.0.0.0' and \
        source_port == 0 and target_port == 0):
      return False
    elif(source_ip == '0.0.0.0' and target_ip == '0.0.0.0'):
      if(source_port == 0 and ( target_port == self.source_port or  target_port == self.target_port)):
        ip_matched = True
        ports_matched = True
      elif(target_port == 0 and ( source_port == self.source_port or  source_port == self.target_port)):
        ip_matched = True
        ports_matched = True
      elif((source_port == self.source_port and  target_port == self.target_port) or \
              (source_port == self.target_port and  target_port == self.source_port)):
        ip_matched = True
        ports_matched = True

    elif(target_ip == '0.0.0.0'):
      if(target_port == 0 and source_ip == self.source_ip and source_port == self.source_port):
        ip_matched = True
        ports_matched = True
      elif(target_port == 0 and source_ip == self.target_ip and source_port == self.target_port):
        ip_matched = True
        ports_matched = True
      elif(target_port == self.target_port and source_ip == self.source_ip and source_port == self.source_port):
        ip_matched = True
        ports_matched = True
      elif(target_port == self.source_port and target_ip == self.source_ip and source_port == self.target_port):
        ip_matched = True
        ports_matched = True

    elif(source_ip == '0.0.0.0'):
      if(source_port == 0 and target_ip == self.target_ip and target_port == self.target_port):
        ip_matched = True
        ports_matched = True
      elif(source_port == 0 and target_ip == self.source_ip and target_port == self.source_port):
        ip_matched = True
        ports_matched = True
      elif(source_port == self.source_port and target_ip == self.target_ip and target_port == self.target_port):
        ip_matched = True
        ports_matched = True
      elif(source_port == self.target_port and target_ip == self.source_ip and target_port == self.source_port):
        ip_matched = True
        ports_matched = True

    elif((self.source_ip == source_ip and \
          self.target_ip == target_ip and \
          self.source_port ==  source_port and \
          self.target_port ==  target_port) or \
          (self.target_ip == source_ip and \
           self.source_ip == target_ip and \
           self.source_port ==  target_port and \
           self.target_port ==  source_port)):
        ip_matched = True
        ports_matched = True
#    print ip_matched, ports_matched
    return (ip_matched and ports_matched)

  def is_packet_in_flow(self, source_ip, source_port, target_ip, target_port, protocol, timestamp):
    return ((self.source_ip == source_ip and \
            self.target_ip == target_ip and \
            self.source_port ==  source_port and \
            self.target_port ==  target_port) or \
            (self.target_ip == source_ip and \
             self.source_ip == target_ip and \
             self.source_port ==  target_port and \
             self.target_port ==  source_port)) and \
           self.protocol ==  protocol and \
           math.fabs(self.timestamp - timestamp) <= self.timeout

  def append_packet_client(self, packet):
    #print "Inserting packet"
    return self.clientpackets.append(packet)

  def append_packet_server(self, packet):
    #print "Inserting packet"
    return self.serverpackets.append(packet)

  def remove_packet(self, packet):
    #print "Removing packet"
    self.clientpackets.remove(packet)
    self.serverpackets.remove(packet)
    return true

  def __str__(self):
    return self.appname + "["+ self.protocol+ "(" + str(len(self.clientpackets)) + ")"+ "(" + str(len(self.serverpackets)) + ")"+"]" + "-" + self.source_ip + ":" + str(self.source_port) + "<=>" + self.target_ip + ":" + str(self.target_port) + " || "\
        + time.strftime("%a, %b %d, %Y %H:%M:%S." + str(self.timestamp - int(self.timestamp)),time.localtime(int(self.timestamp))) + "\n" + "-----------------------------------------------"  * 2


# This class deals with the list of flows
class FlowsList:
  def __init__(self):
    self.flows = []

  def merge_flows(self, f1, f2):
    #print "Merging flows"
    b1 = f1.clientpackets.extend(f2.clientpackets)
    b2 = f1.serverpackets.extend(f2.serverpackets)
    f1.starttimestamp = min(f1.starttimestamp, f2.starttimestamp)
    f1.timestamp = max(f1.timestamp, f2.timestamp)
    del f2
    return b1 and b2

  def append_flow(self, nf):
    #print "appending flows"
    for fl in self.flows:
      if fl.is_packet_in_flow(nf.source_ip, nf.source_port, nf.target_ip, nf.target_port, nf.protocol, nf.timestamp):
        return self.merge_flows(fl, nf)
    return self.flows.append(nf)

  def remove_flow(self, nf):
    #print "removing flows"
    return self.flows.remove(nf)

  def __str__(self):

    retval = ""
    for fl in self.flows:
       retval += str(fl) + "\n"

    return retval
